/***************************************************************************
 *                                                                         *
 *   timecard.cpp - Program to parse my engineering journal for time info  *
 *                                                                         *
 *   Copyright (C) 2005 by G. M. Beddingfield <gabriel@teuton.org>         *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*
 * timecard.cpp
 * G. M. Beddingfield
 * 18MAR2005
 *
 * Program to parse my html journal, pull out the <h3> lines
 * that contain log data, and print a report that has those with
 * a decimal elapsed time.
 */

#include <iostream>
#include <fstream>
#include <iomanip>
#include <sstream>
#include <string>
#include <vector>
#include <map>
#include <cctype>

using namespace std;

class LogEntry;
typedef vector<LogEntry> Logs;

class LogEntry
{
	string _l; // text of log line (slightly modified)
	string _a; // account name of log entry
	double _t; // time (military, decimal)
protected:
	void strip_html(string& x);
	void strip_acct(void);

public:
	LogEntry(string l);
	LogEntry(const LogEntry& x) : _l(x._l), _t(x._t), _a(x._a) { }
	~LogEntry() { }

	inline double time(void) const { return _t; }
	inline string acct(void) const { return _a; }
	friend ostream& operator<<(ostream& os, const LogEntry& x);
};

LogEntry::LogEntry(string l) : _l(l), _t(0), _a("NULL")
{
	size_t n;

	// Remove the html <> tags and get account number
	strip_html(_l);
	strip_acct();

	n = _l.find(':');
	if( n == string::npos ) return;

	// Otherwise convert the surrounding to decimal hours
	char s[5];
	_l.copy( s, 2, n-2 );
	_t = atof(s);
	_l.copy( s, 2, n+1 );
	_t += atof(s)/60.0;

	// And delete the time from the string
	size_t m;
	n = _l.find('-');
	if( n!=string::npos ) {
		m = _l.find(' ',n);
		if( m!=string::npos ) {
			_l.erase(0,m);
		} else {
			_l.erase(0,n);
		}
	}

	// Strip leading whitespace
	n=0;
	while( isspace(_l[n]) ) {
		_l.erase(n,1);
	}

	return;
}

void LogEntry::strip_html(string& x)
{
	size_t p,q;

	while( (p=x.find('<')) != string::npos ) {
		q = x.find('>');
		if( (q>p)||(q==string::npos) ) {
			x.erase(p,++q);
		} else {
			break;
		}
	}
	return;
}

void LogEntry::strip_acct(void)
{
	// A typical log entry looks like this:
	// <h3>08:30 UNBS012345A3 -- This is what I did.</h3>
	// Between the time and the first dash is the account number

	size_t p,q;

	p=_l.find(':');
	if( p!=string::npos ) p = _l.find(" ",p);
	q=_l.find('-');
	if( q!=string::npos ) q = _l.rfind(" ",q);

	if( (p!=string::npos)&&(q!=string::npos)&&(q>p) ) {
		q -= p;
		_a = _l.substr(p,q);
		_l.erase(p,q);
	}
	p=0;
	while( p<_a.size() ) {
		if(isspace(_a[p])) {
			_a.erase(p,1);
		} else {
			++p;
		}
	}

	return;
}

ostream& operator<<(ostream& os, const LogEntry& x)
{
	os << setw(5) << setiosflags(ios::fixed) << setprecision(2)
	   << showpoint << x._t << " " << setw(18) << x._a << " " << x._l ;
	return os;
}

inline double operator+(const LogEntry& a, const LogEntry& b)
{
	return a.time() + b.time();
}

inline double operator-(const LogEntry& a, const LogEntry& b)
{
	return a.time() - b.time();
}

ostream& operator<<(ostream& os, const Logs& x)
{
	static const double eps = .001;
	typedef map<string,double> totalizer;
	totalizer sums;

	os << "TASK   TIME   ACCOUNT NUMBER   DESCRIPTION " << endl;
	os << "===== ===== ================== ======================"
		"======" << endl;
	int j;
	double d, sum=0.0;
	for( int n=0 ; n < x.size() ; n++ ) {
		// Find next non-zero time in array.
		if( fabs(x[n].time())>eps ) {
			for( j=n+1 ; (j<x.size()) && (fabs(x[j].time())<eps) ;
			     ++j );
			// j now points to next time.
			d = ( j<x.size() )?( x[j].time()-x[n].time() ):( 0.0 );
		} else {
			d = 0.0;
		}
		sum += d;
		sums[x[n].acct()] += d;
		os << setw(5) << setiosflags(ios::fixed)
		   << setprecision(2) << d << " " << x[n] << endl;
	}
	os << "=====" << endl
	   << setw(5) << setiosflags(ios::fixed) << setprecision(2)
	   << sum << " Total (including breaks)" << endl;

	os << endl << endl;
	os << "Summaries by account:" << endl;
	os << "---------------------" << endl;
	os << endl;

	totalizer::iterator si;
	sum = 0;
	for( si=sums.begin(); si!=sums.end(); ++si ) {
		os << setw(5) << setiosflags(ios::fixed) << setprecision(2)
		   << (*si).second << " " << (*si).first << endl;
		if( (*si).first!=string("NULL") ) sum += (*si).second;
	}
	os << "=====" << endl
	   << setw(5) << setiosflags(ios::fixed) << setprecision(2)
	   << sum << " Total (not including account NULL)" << endl;

	cout << endl;

	return os;
}


int main(int argc, char* argv[])
{
	if( argc<2 ) {
		cout << "Need to specify filename." << endl;
		return EXIT_SUCCESS;
	}

	ifstream f(argv[1]);
	if(f.bad()) {
		cout << "Error:  could not open file."  << endl;
		return EXIT_FAILURE;
	}

	string line;
	Logs logs;
	int pos;
	LogEntry *tmp;

	while( getline( f, line ) ) {

		// Search for "<h3" or "<H3"
		pos = line.find("<h3");
		if( pos == string::npos ) {
			pos = line.find("<H3");
		}
		if( pos == string::npos ) continue;

		tmp = new LogEntry(line);
		logs.push_back( *tmp );
		delete tmp;
	}

	cout << endl;
	cout << "................................................"
	     "..........." << endl;
	cout << "Time data for " << argv[1] << endl;
	cout << "................................................"
	     "..........." << endl;
	cout << logs;

	return EXIT_SUCCESS;
}
